import {Component} from '@angular/core';
import {SearchCustomFilter} from "../../../openaireLibrary/searchPages/searchUtils/searchUtils.class";
import {ActivatedRoute, Router} from "@angular/router";
import {StakeholderService} from "../../../openaireLibrary/monitor/services/stakeholder.service";

@Component({
  selector: 'monitor-search-results',
  template: `
    <div id="page_content">
      <search-research-results *ngIf="initialized" resultType="result"
                               [customFilter]=customFilter [hasPrefix]="false"
                               [includeOnlyResultsAndFilter]="false"
                               [showSwitchSearchLink]="true"></search-research-results>
    </div>
  `,
})
export class MonitorSearchResearchResultsComponent {
  customFilter: SearchCustomFilter = null;
  initialized: boolean = false;
  
  constructor(private route: ActivatedRoute,
              private router: Router,
              private stakeholderService: StakeholderService) {
  }
  
  ngOnInit() {
    this.route.params.subscribe(params => {
      if (params['stakeholder']) {
        this.stakeholderService.getStakeholderAsObservable().subscribe(stakeholder => {
          if (stakeholder) {
            if (stakeholder.type === "funder") {
              let value = stakeholder.index_id+"||"+stakeholder.index_name+"||"+stakeholder.index_shortName;
              this.customFilter = new SearchCustomFilter("Funder", "relfunder", value, stakeholder.name);
              this.customFilter.isHiddenFilter = false;
              this.customFilter.promptToAddFilter = true;
            } else if (stakeholder.type === "organization") {
              let value = stakeholder.index_id;
              this.customFilter = new SearchCustomFilter("Organization", "relorganizationid", value,  stakeholder.name);
              this.customFilter.isHiddenFilter = false;
              this.customFilter.promptToAddFilter = true;
            } else if (stakeholder.type === "ri") {
              let value =  stakeholder.index_id+"||"+stakeholder.index_name;
              this.customFilter = new SearchCustomFilter("Community", "community", value,  stakeholder.name);
              this.customFilter.isHiddenFilter = false;
              this.customFilter.promptToAddFilter = true;
            }
            this.initialized = true;
          }
        });
      }
    });
  }
}
